@@ -18,12 +18,12 @@
 // | Authors: Martin Jansen <mj@php.net>                                  |
 // +----------------------------------------------------------------------+
 //
-// $Id: Auth.php,v 1.4 2003/03/13 14:34:12 noriko Exp $
+// $Id: Auth.php,v 1.5 2003/03/24 06:31:53 tumf Exp $
 //
 
 // 
 //  =========================================================================
-// $Date: 2003/03/13 14:34:12 $
+// $Date: 2003/03/24 06:31:53 $
 // This file is modified by Dino Co.,Ltd.from Auth.php
 // 
 // $ cat Auth.php \
@@ -401,12 +401,16 @@ class Auth
          * output the login screen again.
          */
         if (!empty($this->username) && !$login_ok) {
-            $this->status = AUTH_WRONG_LOGIN;
+	  $this->status = AUTH_WRONG_LOGIN;
         }
 
         if ((empty($this->username) || !$login_ok) && $this->showLogin) {
-            $this->drawLogin($this->storage->activeUser);
-            return;
+	  if(strlen($this->username)>0){
+	    $this->drawLogin($this->username);
+	  }else{
+	    $this->drawLogin($this->storage->activeUser);
+	  }
+	  return;
         }
     }
 
