@@ -1,10 +1,11 @@
 <?php {
   /*
-   * $Id: dino_browser.php,v 1.22 2003/06/02 09:07:13 tishizak Exp $
+   * $Id: dino_browser.php,v 1.25 2003/11/11 05:07:56 kasugai Exp $
    * Copyright (c) 2002-2003 Dino Co.,Ltd.
    */
-
-  define('BROWSER_PRIFLE_DIR','browser/profile/');
+  if(BROWSER_PROFILE_DIR=="BROWSER_PROFILE_DIR"){
+    define('BROWSER_PROFILE_DIR',DINOLIB_DIR."/".'browser/profile/');
+  }
   /**   
    * 桼Ƚ̽
    * 
@@ -159,7 +160,7 @@
      * @return boolen üʤtrue, Or false
      */
     function is_handy(){
-      if ( $this->get_terminal_type() != "etc" ) {
+      if ( Dino_Browser::get_terminal_type() != "etc" ) {
 	return true;
       }
     }
@@ -225,11 +226,12 @@
 	$disp["depth"]  = getenv("HTTP_X_UP_DEVCAP_SCREENDEPTH");  //
 
       } elseif ( $term == "docomo" ) { //DoCoMo 
-	require_once(BROWSER_PROFILE_DIR."i_display.inc"); //ɽǽϥǡ
+	//require_once(BROWSER_PROFILE_DIR."i_display.inc.php"); //ɽǽϥǡ	
+	$imode_display = $this->get_display_data();
 	$imode = $this->get_basic_info();
-
-	list( $disp["width"], $disp["height"], $disp["depth"] ) = $imode_display[$imode["name"]];
-	$disp["color"]  = ( $disp["depth"] >= 8 ); //8ʾǥ顼
+	list( $disp["width"], $disp["height"], $depth ) = $imode_display[$imode["name"]];
+	$disp["depth"]=pow(2,$depth);
+	$disp["color"]  = ( $depth >= 8 ); //8ʾǥ顼
 
       } else {
 	return false;
@@ -237,6 +239,22 @@
       return $disp;
     }
 
+    //CSVΥǡե뤫ǡ
+    function get_display_data(){
+      $path = BROWSER_PROFILE_DIR."i_display.inc";
+      if(!file_exists($path)){
+	echo $path."¸ߤޤ<br>";
+      }else{
+	$f = fopen($path,"r");
+	while($data = fgetcsv($f,1000)){
+	  $r[$data[0]]=array($data[1],$data[2],$data[3]);
+	}
+      }
+      fclose($f);
+      return $r;
+    }
+    
+
     /**
      * ü[]ɽǽ
      *
@@ -255,9 +273,11 @@
 
       } elseif ( $this->terminal == "ez" ) {
 	//case:ez  chord=²type=(0=none/1=mono/2=C-MIDI/3=SMAF)
-	require_once(BROWSER_PROFILE_DIR."ez_chaku.inc"); //ǡ
-	require_once(BROWSER_PROFILE_DIR."ez_qcelp.inc"); //Qcelpбüꥹ
-
+	//require_once(BROWSER_PROFILE_DIR."ez_chaku.inc"); //ǡ	
+	//require_once(BROWSER_PROFILE_DIR."ez_qcelp.inc"); //Qcelpбüꥹ
+	$chaku_data=$this->get_sound_data("ez");
+	$chaku_list = $chaku_data["chaku_list"];
+	$qcelp_list = $chaku_data["qcelp_list"];
 	$info = $this->get_basic_info();
 	$chaku["qcelp"] = "none";
 	if(array_search($info["name"],$qcelp_list)){
@@ -269,8 +289,8 @@
 
       } elseif ( $this->terminal == "docomo" ) {
 	//case:docmo //chord=²,type=(mld,rng)
-	require_once(BROWSER_PROFILE_DIR."i_chaku.inc"); //ǡ
-
+	//require_once(BROWSER_PROFILE_DIR."i_chaku.inc"); //ǡ
+	$chaku_list=$this->get_sound_data("i");
 	$info = $this->get_basic_info();
 	list($chaku["chord"],$chaku["type"])= $chaku_list[$info["name"]];
 
@@ -280,6 +300,59 @@
       return $chaku;
     }
 
+    //CSVΥǡե뤫ǡ
+    function get_sound_data($carrier){
+      if($carrier=="ez"){
+	$file_chaku = BROWSER_PROFILE_DIR."ez_chaku.inc";
+	$file_qcelp = BROWSER_PROFILE_DIR."ez_qcelp.inc";
+	$file_chaku_flg = true;
+	$file_qcelp_flg = true;
+	if(!file_exists($file_chaku)){
+	  echo $file_chaku."¸ߤޤ<br>";
+	  $file_chaku_flg = false;
+	}
+	if(!file_exists($file_qcelp)){
+	  echo $file_chaku."¸ߤޤ<br>";
+	  $file_qcelp_flg = false;
+	}
+
+	if($file_chaku_flg&&$file_qcelp_flg){
+	  $f = fopen($file_chaku,"r");
+	  while($chaku_data = fgetcsv($f,1000)){
+	    $r["chaku_list"][$chaku_data[0]]=array($chaku_data[1],$chaku_data[2]);
+	  }
+	  
+	  $f2 = fopen($file_qcelp,"r");
+	  while($qcelp_data = fgetcsv($f2,1000)){
+	    $r["qcelp_list"]=$qcelp_data;
+	  }
+	  fclose($f);
+	  fclose($f2);
+	  return $r;
+	}else{
+	  return false;
+	}
+      }elseif($carrier=="i"){
+	$file_chaku = BROWSER_PROFILE_DIR."i_chaku.inc";
+	
+	if(!file_exists($file_chaku)){
+	  echo $file_chaku."¸ߤޤ<br>";
+	  return false;
+	}else{
+	  $f = fopen($file_chaku,"r");
+	  while($data = fgetcsv($f,1000)){
+	    $r[$data[0]]=array($data[1],$data[2]);
+	  }
+	  fclose($f);
+	  return $r;
+	}
+      }else{
+	echo "ꥢоݳǤ";
+	return false;
+      }
+    }
+
+
     /**
      * üδܾ
      *
@@ -296,7 +369,8 @@
     function get_basic_info(){
       if($this->terminal == "jphone" ){
 	//J-Phone by vodafone
-	require_once(BROWSER_PROFILE_DIR."j_name.inc"); //̾ǡ
+	//require_once(BROWSER_PROFILE_DIR."j_name.inc"); //̾ǡ
+	$jphone_name = $this->get_basic_data($this->terminal);
 
 	$jphone = getenv("HTTP_X_JPHONE_MSNAME"); //̾(+version)
 	$arrAgent = split("/",$this->agent);
@@ -307,8 +381,8 @@
 
       } elseif ( $this->terminal == "ez" ) {
 	//au by KDDI
-	require_once(BROWSER_PROFILE_DIR."ez_name.inc"); //̾ǡ
-
+	//require_once(BROWSER_PROFILE_DIR."ez_name.inc"); //̾ǡ
+	$ezweb_name = $this->get_basic_data($this->terminal);
 	$piece = split("/",$this->agent);
 	if( ereg("^KDDI\-([^ ]+)",$piece[0],$regarr) ){
 	  $name = $regarr[1];
@@ -334,10 +408,9 @@
 
       } elseif ($this->terminal == "docomo" ) {
 	//NTT DoCoMo
-	require_once(BROWSER_PROFILE_DIR."i_ver.inc");
-
+	//require_once(BROWSER_PROFILE_DIR."i_ver.inc");
+	$imode_ver = $this->get_basic_data($this->terminal);
 	list( $docomo, $ver, $name, $sub) = split( "/", $this->agent );
-
 	$info["type"]   = "i";   //DoCoMo
 	$info["name"]   = $name;
 	$info["sub"]    = ($sub == "")?"c5":$sub;  //cache :default 5k
@@ -349,6 +422,28 @@
       return $info;
     }
 
+    function get_basic_data($carrier){
+      if($carrier = "jphone"){
+	$path = BROWSER_PROFILE_DIR."j_name.inc";
+      }elseif($carrier = "ez"){
+	$path = BROWSER_PROFILE_DIR."ez_name.inc";
+      }elseif($carrier = "docomo"){
+	$path = BROWSER_PROFILE_DIR."i_ver.inc";
+      }else{
+	return false;
+      }
+      if(!file_exists($path)){
+	echo $path."¸ߤޤ<br>";
+	return false;
+      }else{
+	$f = fopen($path,"r");
+	while($data = fgetcsv($f,1000)){
+	  $r[$data[0]]=$data[1];
+	}
+	fclose($f);
+	return $r;
+      }
+    }
 
     /**
      * ʸɤ
