@@ -1,6 +1,6 @@
 <?php {
   /*
-   * $Id: dino_db.php,v 1.22 2003/05/13 05:58:20 tishizak Exp $
+   * $Id: dino_db.php,v 1.24 2003/10/06 05:52:52 shota Exp $
    * Copyright (c) 2002-2003 Dino Co.,Ltd.
    */
 
@@ -76,9 +76,21 @@
 	if($affected = $this->affectedRows()) {
 	  $this->logging_sql('query()', $sql, $affected);
 	} else {
-	  $rs =& new Dino_Result(check_db_error($this->db->query($sql)));
-	  $this->logging_sql('query()', $sql, $rs->numRows());
-	  return $rs;
+	  $result = $this->db->query($sql);
+	  if ($result === DB_OK) {
+	    // do nothing
+	  } else {
+	    $rs =& new Dino_Result(check_db_error($result));
+	    $this->logging_sql('query()', $sql, $rs->numRows());
+	    // LOGGING trueˤȥ顼Ф롣
+	    //Fatal error: Call to a member function on a non-object in 
+	    // /usr/local/share/dinolib/lib/db/dino_db_result.php on line 61
+	    // $rs->numRows() 
+	    // PEARquery()֥Ȥ֤ʤȤ
+
+	    return $rs;
+	  }
+
 	}
       }
       return new Dino_Result(check_db_error($this->db->query($sql)));
@@ -280,12 +292,11 @@
       //$linenum = __LINE__;
       //$linenum = $this->db->quote($linenum);
       if(is_null($result)) $result = 'null';
-      check_db_error($this->db->query("
-	insert into __logging_sql
-	  (func_name, query, result_option, called_file)
-	values
-	  ($func, $sql, $result, $fname)
-      "));
+      else $result = $this->db->quote($result);
+      $logsql = "insert into __logging_sql(func_name, query, result_option, called_file) values($func, $sql, $result, $fname)";
+#      echo $logsql;
+ #     echo "<br>\n";
+      check_db_error($this->db->query($logsql));
     }
 
     /**
