@@ -1,6 +1,6 @@
 <?php {
   /*
-   * $Id: dino_form.php,v 1.45 2003/07/10 03:19:22 tumf Exp $
+   * $Id: dino_form.php,v 1.49 2003/09/19 02:38:46 hanawa Exp $
    * Copyright (c) 2002-2003 Dino Co.,Ltd.
    */
 
@@ -21,8 +21,8 @@
       date5 --> datei
       date6 --> dates
   */
-  require_once("form/tpl_form.inc.php");
-  require_once("form/oohforms.inc.php");
+  require_once(DINOLIB_DIR . "/form/tpl_form.inc.php");
+  require_once(DINOLIB_DIR . "/form/oohforms.inc.php");
 
   class Dino_form extends tpl_form {
     // tplvar_form_fields() ǡ̾ΥեǤget_element()롣
@@ -191,7 +191,6 @@
       return true;
     }
     function pre_disp() {
-      // global $form_name;
       $form_name = request_var("form_name");
       if (! is_object($this->form_data)) {
 	$this->setup();
@@ -298,6 +297,13 @@
       return $this->form_data->get_element($name,$num);
     }
 
+    function freeze($flist=""){
+      return $this->form_data->freeze($flist);
+    }
+    function hide($flist=""){
+      return $this->form_data->hide($flist);
+    }
+
 
 
 
@@ -327,9 +333,14 @@
       $this->set_default_values_from_assoc($values);
     }
     
-    // DB ==> form
-    //
-    // Τޤ
+    /**
+     * DBäƤͤformͤ˰ΧѴ
+     * 
+     * values[]ѿ̾=ͤ褿ˡ
+     * $fv["ѿ̾"] = ""
+     * 
+     * @access public
+     */
     function values2fv_all(&$values, &$fv, $suffix = "") {
       reset($values);
       while (list($k, $v) = each($values)) {
@@ -341,15 +352,16 @@
 	}
       }
     }
+
     /**
-     *  DB ==> form
+     * DBäƤͤformͤѴ
      * 
-     *  (餯)DBäƤͤ set_default_values() Ϥ褦ˤ
-     *  (ٹɬפʤȤ⤢Ϥ/selectʸǺٹƤɤ)
+     * (餯)DBäƤͤ set_default_values() Ϥ褦ˤ
+     * (ٹɬפʤȤ⤢Ϥ/selectʸǺٹƤɤ)
      * 
-     *  overrideƻȤ
-     *  $valuesϽ񤭴ᡣ
-     *  $fvϾOK
+     * overrideƻȤ
+     * $valuesϽ񤭴ᡣ
+     * $fvϾOK
      *
      * @access public
      */
@@ -373,12 +385,12 @@
       return $this->values;
     }
     /**
-     *  form ==> DB
+     * formͤDB뤿ͤ˰ΧѴ
      *
-     *  queryȤѿ̾=ͤ褿ˡ
-     *  $values["ѿ̾"] = ""
-     *  ѿ̾ѿ̾1, ѿ̾2Ĥʤɤξ
-     *  $values[1]["ѿ̾"] = ""ʤɤȤ롣
+     * queryȤѿ̾=ͤ褿ˡ
+     * $values["ѿ̾"] = ""
+     * ѿ̾ѿ̾1, ѿ̾2Ĥʤɤξ
+     * $values[1]["ѿ̾"] = ""ʤɤȤ롣
      *
      * @access public
      */
@@ -398,6 +410,34 @@
 	}
       }
     }
+    /**
+     * formͤDB뤿ͤ˰ΧѴ
+     *
+     * queryȤѿ̾=ͤ褿ˡ
+     * $values["ѿ̾"] = ""
+     * ѿ̾ѿ̾1, ѿ̾2Ĥʤɤξ
+     * $values[1]["ѿ̾"] = ""ʤɤȤ롣
+     *
+     * ƱȾѥʢѥʡѱѿȾѱѿѴԤ
+     *
+     * @access public
+     */
+    function fv2values_all_normalize(&$fv, &$values, $suffix_flag = false) {
+      reset($fv);
+      if ($suffix_flag) {
+	while (list($k, $v) = each($fv)) {
+	  if (ereg("^(.*[^0-9])([0-9]+)$", $k, $regs)) {
+	    $values[$regs[2]][$regs[1]] = mb_convert_kana($v, "KVa");
+	  } else {
+	    $values[$k] = mb_convert_kana($v, "KVa");
+	  }
+	}
+      } else {
+	while (list($k, $v) = each($fv)) {
+	  $values[$k] = mb_convert_kana($v, "KVa");
+	}
+      }
+    }
 
     /**
      *  form ==> DB
