@@ -1,6 +1,6 @@
 <?php {
   /*
-   * $Id: dino_sql.php,v 1.86 2003/06/25 07:25:22 kasugai Exp $
+   * $Id: dino_sql.php,v 1.121 2003/11/27 13:39:07 noriko Exp $
    * Copyright(c) 2001-2003 Dino Co.,Ltd. All rights reserved.
    */
 
@@ -11,7 +11,7 @@
    * Υ饹Ѿơǡ١ΰĤTABLEˤĤ
    * Ĥζݥ饹ޤ
    * ʣΥơ֥ˤޤ褦SQLꤿˤϡ
-   * Ȥʤơ֥Υ饹Ƥ֤褦ˤޤ
+   * SQL֥ȤФ̤SQL֥Ȥjoin뤳ȤǤޤ
    *
    * 㤨Сtestơ֥ˤĤƤΥ饹Test_SQLˤĤơ
    * 
@@ -32,32 +32,44 @@
    * 
    * (1)SQLҤɸಽ
    * 
-   * SQLʸϰ̤80ʸĶʸˤʤꡢ
-   * ʬʤɤSQLʸʸ˳ǼˡϿͤ줾Ǥ
-   * Τ褦ʥɤʣͿǤΥƥʥޤ
+   * ץSQLʸ80ʸĶΤ¿ɤߤˤΤǤ
+   * SQL˴ؤƥɸƤȤäʹȤޤ
+   * ޤʬSQLʸʸ˳ǼˡϿͤ줾Ǥ
+   * ˳Ǽ뤳Ȥ⤢СʸϢ뤹뤳Ȥ⤢Ǥ礦
+   * Τ褦ˡSQLʸľܽ񤤤Ƥȡ
+   * ʣͿǤγȯ˥ƥʥ򲼤ǽޤ
+   * Υ饹ȤɸಽǽǤ
    * 
    * (2)SQLФϤSQLʸˡ顼ˤʤꥹ㸺
    *
    * ưSQLʸ뤿ᡢդ䥨顼å­ˤä
-   * SQLǽưǺ٤ƾ
+   * SQLǽưǺ꾮Ȥħޤ
    *
-   * ä˾ˤäwhere­­ʤäꤹ
-   * SQL顼ˤʤꥹ򸺤餻ϤǤ롣
+   * äˡ͡ȹ礻where­­ʤäꤹˤϡ
+   * SQL顼ˤʤꥹ򸺤餻ϤǤ
    *
    * ޤƥơ֥бݥ饹ơ֥λĥեɤ
-   * ̾䷿ξäƤ뤳ȤSQLɽˤʤ롣
+   * ̾䷿ξäƤꡢ
+   * ƥ᥽åɤ礬ʤɤåԤᡢ
+   * ե̾δְ㤤ʤɤΥ쥹ߥɤȤǤޤ
    * 
    * (3)桼٥롿˱SQL֤뤳
    * 
-   * 㤨С̥桼 class SQL_tableA_user extends Dino_SQL ʥ饹
-   * Ԥʤ  class SQL_tableA_admin extends SQL_tableA_user ʥ饹
-   * Ƥ֤褦ˤ줾˵줿SQLΤߤ֤褦ˤޤ
+   * ݥ饹ǡեɤȤinsertupdateβ/ԲĤǤΤǡ
+   * 桼Ȥ˰SQLȯԤݾڤǤ褦ˤʤޤ
+   *
+   * 㤨С̥桼 class SQL_tableA_user extends Dino_SQL 
+   * Ǥ륯饹
+   * Ԥʤ class SQL_tableA_admin extends SQL_tableA_user 
+   * ǤƤ֤褦ˤ줾˵줿SQLΤߤ֤褦ˤ
+   * ԤΤߤǤեɤSQL_tableA_adminˤΤ
+   * Ҥ뤳ȤǡSQLʸϤ뤳ȤǤޤ
    *
    *
    * ǰʤ顢Υ饹SQLʸǤäƤ
    * Ф˥顼ФʤȤݾڤϤޤ
-   * ʤȤƥơ֥뤴ȤSQL饹ǤɬפϤޤ
-   * ξϡƤӽФ¦ΥɤǤΥ顼åɬפǤ礦
+   * ʤȤƥơ֥бSQL饹ǤɬפϤޤ
+   * ξϡƤӽФ¦ΥɤǤ⥨顼åɬפǤ礦
    * ȤϤߥ򸺤餹ˤϤʤȹͤƤޤ
    *
    * todo:
@@ -66,7 +78,60 @@
    *@access public
    *@package Dino_SQL
    */
-  class Dino_SQL {
+  
+  // {{{ constants
+  // {{{ error codes
+  /*
+   * see DB.php
+   */
+  
+  define("SQL_OK",                         1);
+  define("SQL_ERROR",                     -1);
+  define("SQL_ERROR_SYNTAX",              -2);
+  define("SQL_ERROR_CONSTRAINT",          -3);
+  define("SQL_ERROR_NOT_FOUND",           -4);
+  define("SQL_ERROR_ALREADY_EXISTS",      -5);
+  define("SQL_ERROR_UNSUPPORTED",         -6);
+  define("SQL_ERROR_MISMATCH",            -7);
+  define("SQL_ERROR_INVALID",             -8);
+  define("SQL_ERROR_NOT_CAPABLE",         -9);
+  define("SQL_ERROR_TRUNCATED",          -10);
+  define("SQL_ERROR_INVALID_NUMBER",     -11);
+  define("SQL_ERROR_INVALID_DATE",       -12);
+  define("SQL_ERROR_DIVZERO",            -13);
+  define("SQL_ERROR_NODBSELECTED",       -14);
+  define("SQL_ERROR_CANNOT_CREATE",      -15);
+  define("SQL_ERROR_CANNOT_DELETE",      -16);
+  define("SQL_ERROR_CANNOT_DROP",        -17);
+  define("SQL_ERROR_NOSUCHTABLE",        -18);
+  define("SQL_ERROR_NOSUCHFIELD",        -19);
+  define("SQL_ERROR_NEED_MORE_DATA",     -20);
+  define("SQL_ERROR_NOT_LOCKED",         -21);
+  define("SQL_ERROR_VALUE_COUNT_ON_ROW", -22);
+  define("SQL_ERROR_INVALID_DSN",        -23);
+  define("SQL_ERROR_CONNECT_FAILED",     -24);
+  define("SQL_ERROR_EXTENSION_NOT_FOUND",-25);
+  define("SQL_ERROR_ACCESS_VIOLATION",   -26);
+  define("SQL_ERROR_NOSUCHDB",           -27);
+  define("SQL_ERROR_NOCONDITIONS",       -28);
+  define("SQL_ERROR_NOSETVAR",           -29);
+  define("SQL_ERROR_OTHERTABLE",         -30);
+  define("SQL_ERROR_NO_WHERE_CLAUSE",    -31);
+  define("SQL_ERROR_NO_PKEY_IN_WHERE",   -32);
+
+  // }}}
+  // {{{ warning codes
+  /*
+   * see DB.php
+   */
+  
+  define('SQL_WARNING',           -1000);
+  define('SQL_WARNING_READ_ONLY', -1001);
+
+  require_once('PEAR.php');
+  //require_once(DINOLIB_DIR . '/sql/dino_sql_error.php');
+  
+  class Dino_SQL extends PEAR {
     var $main_table = "";
     var $table = array(); // obsolete
 
@@ -130,6 +195,55 @@
     */
     var $field_labels = array();
 
+   /**
+    * primary keyȤƻꤵƤե̾
+    *
+    *@see get_pkey_name()
+    */
+    var $pkey_name = array();
+
+   /**
+    * ǧڥơ֥ΥȤʤäƤե̾
+    *
+    *@see get_akey_name()
+    */
+    var $akey_name = "";
+
+   /**
+    * foregin keyȤƻꤵƤե̾
+    *
+    *@see get_fkey_name()
+    */
+    var $fkey_name = array();
+
+   /**
+    * ǧڥơ֥뤬Ĥˤforegin key
+    *
+    */
+    var $fkey_auth_ancestor = "";
+
+    /**
+     * 顼⡼
+     *
+     * @var     int
+     * @access  private
+     */
+    var $error_mode = null;
+
+    /**
+     * PEAR_ERROR_TRIGGER ѤΥ顼ץ
+     *
+     * @var     int
+     * @access  private
+     */
+    var $error_options = null;
+
+    /**
+     * SQL object
+     * @access private
+     */
+    var $sql;
+
     /**
      * 󥹥ȥ饯
      *
@@ -138,10 +252,12 @@
      */
     function Dino_SQL($alias = "") {
       $this->initialize();
-
+      
       if ($alias) {
 	$this->set_alias($alias);
       }
+
+      $this->PEAR('SQL_Error');
     }
 
     /**
@@ -161,6 +277,24 @@
       $this->groupby = "";
     }
 
+    function &raiseError($code = SQL_ERROR, $mode = null, $options = null,
+                         $userinfo = null, $nativecode = null)
+      {
+	// The error is yet a SQL error object
+	if (is_object($code)) {
+	  // because we the static PEAR::raiseError, our global
+	  // handler should be used if it is set
+	  if ($mode === null && !empty($this->_default_error_mode)) {
+	    $mode    = $this->_default_error_mode;
+	    $options = $this->_default_error_options;
+	  }
+	  return parent::raiseError($code, null, $mode, $options, null, null, true);
+        }
+	
+        return parent::raiseError(null, $code, $mode, $options, $userinfo,
+                                  'SQL_Error', true);
+    }
+
     /**
      * ơ֥Υꥢ̾ꤹ
      *
@@ -233,6 +367,48 @@
       return implode(" and ", $this->where);
     }
 
+    /**
+     * primary keyΥե֤̾
+     *
+     *@return string primary keyΥե̾
+     */
+    function get_pkey_name(){
+      return $this->pkey_name;
+    }
+
+    /**
+     * ǧkeyΥե֤̾
+     *
+     *@return string ǧkeyΥե̾
+     */
+    function get_akey_name(){
+      return $this->akey_name;
+    }
+
+    /**
+     * foreign keyΥե֤̾
+     *
+     *@return string foreign fieldΥե̾
+     */
+    function get_fkey_name(){
+      return $this->fkey_name;
+    }
+
+    /**
+     * ƥơ֥SQL֥Ȥ֤
+     *
+     *@param $fkey foreign key
+     *@return object ƥơ֥SQL֥/顼ξfalse
+     */
+    function get_ptbl_obj($fkey){
+      return false;
+    }
+
+    //function get_akey_name(){
+    //  return $this->akey_name;
+    //}
+
+
     function traverse_sql_tree(&$select_array, &$from_array,
 			       &$where_array, &$orderby_array,
 			       $is_used = array()) {
@@ -319,7 +495,9 @@
     *@param string $label ٥(ɽ٤)
     */
     function set_labels($fname, $value, $label) {
-      $this->field_labels[$fname][$value]=$label;
+      if($this->field_exists($fname)){
+	$this->field_labels[$fname][$value]=$label;
+      }
     }
 
    /**
@@ -328,12 +506,15 @@
     *@see set_labels()
     */
     function get_labels($fname) {
-      $a = $this->field_labels[$fname];
-      if (is_array($a)) {
-	return $a;
-      } else {
-	return array();
+      if($this->field_exists($fname)){
+	$a = $this->field_labels[$fname];
+	if (is_array($a)) {
+	  return $a;
+	} else {
+	  return array();
+	}
       }
+      return array();
     }
 
     // Υ饹selectʤɤƤӽФݤɬפwhereʤɤå
@@ -362,6 +543,30 @@
     function set_condition($field, $value) {
       return $this->set_condition_op($field, "=", $value);
     }
+    /**
+     *ѿץ饤ޥꥭ̾
+     *֡ĤϡĤ()ꤹ
+     *
+     *ץ饤ޥꥭʣarray
+     */
+    function set_condition_pkey($value) {
+      //ϥå
+      if(is_array($value)){
+	if(!is_array($this->pkey_name)){
+	  echo "set_condition_pkey: ץ饤ޥꥭʣޤ";
+	  exit;
+	}else{
+	$ok = false;
+	foreach($this->pkey_name as $key => $pkey_name){
+	  if($this->set_condition_op($pkey_name, "=", $value[$key]))
+	    $ok = true;
+	}	     
+	return $ok;
+	}
+      }else{
+	return $this->set_condition_op($this->pkey_name, "=", $value);
+      }
+    }
     /**
      * ֡ĤϡĤ()ꤹ
      *
@@ -490,8 +695,11 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_condition_null($field) {
-      $this->where[$field] = $field. " is null";
-      return 0;
+      if ($this->field_exists($field)) {
+	$this->where[$field] = $field. " is null";
+	return 0;
+      }
+      return -1;
     }
     /**
      * ֡Ĥnullפꤹ
@@ -514,8 +722,11 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_condition_is_not_null($field) {
-      $this->where[$field."_is_not_null"] = $field. " is not null";
-      return 0;
+      if($this->field_exists($field)){
+	$this->where[$field."_is_not_null"] = $field. " is not null";
+	return 0;
+      }
+      return -1;
     }
 
     /**
@@ -532,15 +743,19 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_condition_like($field, $value, $esc = "") {
-      if (isset($value)) {
-	if (isset($esc)) {
-	  $this->where[$field."_like"] = sprintf("%s like '%s' escape '%s'",
-						 $field, (string)$value, $esc);
-	} else {
-	  $this->where[$field."_like"] = sprintf("%s like '%s'",
-						 $field, (string)$value, $esc);
+      if($this->field_exists($field)){
+	if (isset($value)) {
+	  if (isset($esc)) {
+	    $this->where[$field."_like"] = sprintf("%s like '%s' escape '%s'",
+						   $field, (string)$value, $esc);
+	  } else {
+	    $this->where[$field."_like"] = sprintf("%s like '%s'",
+						   $field, (string)$value, $esc);
+	  }
+	  return 0;
 	}
-	return 0;
+	$this->raiseError(SQL_ERROR_NOSETVAR,null,null,"No set variable for the condition about '$field'",null);
+	return -1;
       }
       return -1;
     }
@@ -557,12 +772,16 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_condition_between($field, $from, $to) {
-      if (isset($from) && isset($to)) {
-	$this->where[$field] = sprintf("%s between %s and %s",
-				       $field,
-				       $this->normalize_value($field, $from),
-				       $this->normalize_value($field, $to));
-	return 0;
+      if($this->field_exists($field)){
+	if (isset($from) && isset($to)) {
+	  $this->where[$field] = sprintf("%s between %s and %s",
+					 $field,
+					 $this->normalize_value($field, $from),
+					 $this->normalize_value($field, $to));
+	  return 0;
+	}
+	$this->raiseError(SQL_ERROR_NOSETVAR,null,null,"No set variable for the condition about '$field'",null);
+	return -1;
       }
       return -1;
     }
@@ -582,44 +801,55 @@
      *@return string 
      */
     function normalize_value($field, $value) {
-      /* SQLʸ˽񤱤褦ˤ뤿ᡢȤ򥨥פ */
-      $value = str_replace ("'", "''", $value);
-      $value = str_replace ('\\', '\\\\', $value);
+      if($this->field_exists($field)){
+	/* SQLʸ˽񤱤褦ˤ뤿ᡢȤ򥨥פ */
+	$value = str_replace ("'", "''", $value);
+	$value = str_replace ('\\', '\\\\', $value);
+	
+	$type = $this->field_type[$field];
+	if (!$type) {
+	  $type = "text";
+	}
 
-      $type = $this->field_type[$field];
-      if (!$type) {
-	$type = "text";
-      }
-      if ($type == "integer") {
-	if (strlen($value) == 0 ||
-	    (((string)$value != "0") && ((int)$value == 0))) {
+	if(strlen($value) == 0){
+	  //valueͿʤΤϥ顼Ф̤
+	  if ($type == "integer" || $type == "serial" || $type == "boolean") {
+	    return "";
+	  } else {
+	    return "''";
+	  }
+	} elseif ($type == "integer") {
 	  // 㥹Ȥ0ˤʤ뤬äݤͤꤵ줿ϤϤ
-	  return "";
-	}
-	return sprintf("%d", (int)$value);
-      } elseif ($type == "serial") {
-	if ((int)$value <= 0) {
-	  // ͰʳϤϤ
-	  return "";
-	}
-	return sprintf("%d", (int)$value);
-      } elseif ($type == "text") {
-	return sprintf("'%s'", $value);
-      } elseif ($type == "boolean") {
-	return sprintf("'%s'", $value);
-      } elseif ($type == "timestamp") {
-	if (strcasecmp($value, "CURRENT_TIMESTAMP") == 0) {
-	  return "CURRENT_TIMESTAMP";
-	} else if (strcasecmp($value, "CURRENT_DATE") == 0) {
-	  return "CURRENT_DATE";
+	  if( ((string)$value != "0") && ((int)$value == 0) ){
+	    $result = $this->raiseError(SQL_ERROR_MISMATCH,null,null,"the type of '$field' does not match '$value'",null);
+	    return "";
+	  }
+	  return sprintf("%d", (int)$value);
+	} elseif ($type == "serial") {
+	  if ((int)$value <= 0) {
+	    // ͰʳϤϤ
+	    $result = $this->raiseError(SQL_ERROR_MISMATCH,null,null,"'$value' must be positive number for serial field '$field'",null);
+	    return "";
+	  }
+	  return sprintf("%d", (int)$value);
+	} elseif ($type == "text") {
+	  return sprintf("'%s'", $value);
+	} elseif ($type == "boolean") {
+	  return sprintf("'%s'", $value);
+	} elseif ($type == "timestamp") {
+	  if (strcasecmp($value, "CURRENT_TIMESTAMP") == 0) {
+	    return "CURRENT_TIMESTAMP";
+	  } else if (strcasecmp($value, "CURRENT_DATE") == 0) {
+	    return "CURRENT_DATE";
+	  } else {
+	    return sprintf("'%s'", $value);
+	  }
 	} else {
 	  return sprintf("'%s'", $value);
 	}
-      } else {
-	return sprintf("'%s'", $value);
       }
     }
-
+    
     /**
      * 黻Ҥȶꤹ
      *
@@ -634,28 +864,31 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_condition_op($field, $op, $value) {
-      $opid_array = array(
-			  "="    => "",
-			  "<>"   => "_ne",
-			  "<"    => "_max",
-			  "<="   => "_max",
-			  ">"    => "_min",
-			  ">="   => "_min",
-			  "like" => "",
-			  );
-      $opid = $opid_array[$op];
-      if (!isset($opid)) {
-	// Τʤ$opξ"="
-	$op = "=";
-      }
-
-      $where = $this->get_where_clause_op($field, $op, $value);
-      if ($where) {
-	$this->where["${field}${opid}"] = $where;
-	return 0;
-      } else {
-	return -1;
+      if($this->field_exists($field)){
+	$opid_array = array(
+			    "="    => "",
+			    "<>"   => "_ne",
+			    "<"    => "_max",
+			    "<="   => "_max",
+			    ">"    => "_min",
+			    ">="   => "_min",
+			    "like" => "",
+			    );
+	$opid = $opid_array[$op];
+	if (!isset($opid)) {
+	  // Τʤ$opξ"="
+	  $op = "=";
+	}
+	
+	$where = $this->get_where_clause_op($field, $op, $value);
+	if ($where) {
+	  $this->where["${field}${opid}"] = $where;
+	  return 0;
+	} else {
+	  return -1;
+	}
       }
+      return -1;
     }
 
     /**
@@ -692,12 +925,16 @@
      *@return string Ŭڤwhere1ġ顼ξ϶ʸ
      */
     function get_where_clause_op($field, $op, $value='') {
-      $value = $this->normalize_value($field, $value);
-      if ($value == "") {
-	return "";
-      } else {
-	return sprintf("%s%s%s", $field, $op, $value);
-      }
+      if ($this->field_exists($field)) {
+	$value = $this->normalize_value($field, $value);
+	if ($value == "") {
+	  return "";
+	} else {
+	  return sprintf("%s%s%s", $field, $op, $value);
+	}
+      } 
+      return "";
+      
     }
 
     /**
@@ -750,12 +987,14 @@
      *
      */
     function set_condition_str($field, $cond, $value="") {
-      if (strlen($value) == 0) {
-	$value = $cond; // "<", ">", etc...
-	$cond = "=";
-      }
-      if (isset($value)) {
-	$this->where[$field] = sprintf("%s%s'%s'", $field, $cond, (string)$value);
+      if($this->field_exists($field)){
+	if (strlen($value) == 0) {
+	  $value = $cond; // "<", ">", etc...
+	  $cond = "=";
+	}
+	if (isset($value)) {
+	  $this->where[$field] = sprintf("%s%s'%s'", $field, $cond, (string)$value);
+	}
       }
     }
 
@@ -772,9 +1011,11 @@
      *
      */
     function set_condition_str_between($field, $from, $to) {
-      if (isset($from) && isset($to)) {
-	$this->where[$field] = sprintf("%s between '%s' and '%s'",
-				       $field, (string)$from, (string)$to);
+      if($this->field_exists($field)){
+	if (isset($from) && isset($to)) {
+	  $this->where[$field] = sprintf("%s between '%s' and '%s'",
+					 $field, (string)$from, (string)$to);
+	}
       }
     }
 
@@ -787,7 +1028,9 @@
      *@access public
      */
     function unset_condition($field) {
-      unset($this->where[$field]);
+      if($this->field_exists($field)){
+	unset($this->where[$field]);
+      }
     }
     /**
      * ľꤹ
@@ -837,9 +1080,9 @@
     }
 
     /**
-     * Ȥõ
+     * Ȥ
      *
-     * order by õޤ
+     * order by ޤ
      *
      *@access public
      */
@@ -885,17 +1128,33 @@
      * limit ꤷޤ
      *
      *@access public
-     *@param string $limit limitե̾
+     *@param string $limit Ϸξ
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_limit($limit) {
-      if ($limit >= 0) {
+      if ($limit > 0) {
 	$this->limit = (int)$limit;
 	return 0;
+      } else {
+	$this->raiseError(SQL_ERROR_INVALID_NUMBER,null,null,"$limit is invalid for the LIMIT-clause", null);
+	return -1;
       }
-      return -1;
     } 
 
+    /**
+     * ̤νϷξ
+     *
+     * limit ޤ
+     *
+     *@access public
+     *@return int ｪλʤ0۾ｪλʤ-1
+     */
+    function unset_limit() {
+      $this->limit = 0;
+      return 0;
+    } 
+
+
     /**
      * ̤νϤˤĤƥեåͤꤹ
      *
@@ -906,16 +1165,35 @@
      *@return int ｪλʤ0۾ｪλʤ-1
      */
     function set_offset($offset) {
-      if ($offset >= 0) {
+    if ($offset >= 0) {
 	$this->offset = (int)$offset;
+	return 0;
+    } else {
+	$this->raiseError(SQL_ERROR_INVALID_NUMBER,null,null,"$offset is invalid for the OFFSET-clause",null);
+	return -1;
       }
+
     }
 
+    /**
+     * եåͤ
+     *
+     * offset ޤ
+     *
+     *@access public
+     *@param string $offset offset ե̾
+     *@return int ｪλʤ0۾ｪλʤ-1
+     */
+    function unset_offset() {
+      return $this->set_offset(0);
+    }
+
+
     // 졢ȤΤåäƤɤġ
     function lock() {
       if (is_array($this->table) && sizeof($this->table) >= 1) {
 	$table = implode(",", $this->table);
-      } else {
+      } else if ($this->field_exists($field)) {
 	$table = $this->main_table;
       }
 
@@ -955,15 +1233,18 @@
      *@param string $fields joinǷ礹ե
      */
     function set_condition_join_using($obj, $fields, $cfields=""){
-
       if(!$cfields) {
 	$cfields = $fields;
       }
       $field = $fields;
       $cfield = $cfields;
-      $this->where["${field}="] = sprintf("%s.%s=%s.%s", 
-					  $this->get_alias(), $field,
-					  $obj->get_alias(), $cfield);
+      if ($this->field_exists($field) && $obj->field_exists($cfield)) {
+	$this->where["${field}="] = sprintf("%s.%s=%s.%s", 
+					    $this->get_alias(), $field,
+					    $obj->get_alias(), $cfield);
+      } else {
+	// 顼д˾夬äƤϤ
+      }
     }
     /**
      * ơ֥η(join on)
@@ -1015,9 +1296,12 @@
       } else if ($field == "") {
 	reset($field_self);
 	  while (list($k, $v) = each($field_self)) {
-	    $this->where[] = sprintf("%s.%s=%s.%s",
-				     $this->main_table, $v,
-				     $table, $v);
+	    //main_tableФƤΤfield¸ߥåǤФ⤦ФƤå򤫤
+	    if($this->field_exists($field)){
+	      $this->where[] = sprintf("%s.%s=%s.%s",
+				       $this->main_table, $v,
+				       $table, $v);
+	    }
 	  }
       } else if (is_array($field_self) && is_array($field) &&
 		 (sizeof($field_self) == sizeof($field))) {
@@ -1025,9 +1309,12 @@
 	reset($field);
 	while (list($k1, $v1) = each($field_self)) {
 	  list($k2, $v2) = each($field);
-	  $this->where[] = sprintf("%s.%s=%s.%s",
-				   $this->main_table, $v1,
-				   $table, $v2);
+	  //main_tableФƤΤfield¸ߥåǤФ⤦ФƤå򤫤
+	  if($this->field_exists($field)){
+	    $this->where[] = sprintf("%s.%s=%s.%s",
+				     $this->main_table, $v1,
+				     $table, $v2);
+	  }
 	}
       }
     }
@@ -1346,7 +1633,10 @@
     }
 
     function var2sqlval($field, &$varstr) {
-      if ((string)$varstr == "\0") {
+      if ($varstr === false) {
+	return "null";
+      } elseif ((string)$varstr === "\0") {
+	// ΥСȤθߴΤᡣ̤ʸȤư
 	return "null";
       } else {
 	return $this->normalize_value($field, $varstr);
@@ -1369,9 +1659,12 @@
      *@return string insertʸʸ󡣥顼ξʸ
      */
 
-    function insert(&$fv) {
-      if (!is_array($fv))            { $this->_debug2('insert()'); return ""; }
-      if (!$this->insert_fields)     { $this->_debug2('insert()'); return ""; }
+    function insert($fv) {
+      if (!is_array(&$fv))       { $this->_debug2('insert()'); return ""; }
+      if (!$this->insert_fields) { $this->_debug2('insert()'); return ""; }
+
+      //insertfieldĤǤ¸ߤƤ뤫ɤΥå
+      $result = $this->field_exists_for_query(array_keys($fv), "insert");
 
       $insert_f = ""; $insert_v = "";
       reset($this->insert_fields);
@@ -1411,6 +1704,8 @@
     /**
      * ˹礦褦updateʸ֤
      *
+     * primary keywhereǻꤵƤʤϥ顼
+     *
      * $fvϢͤĤ褦ʥ쥳ɤˤʤ褦
      * updateʸ֤
      * set_condition()ʤɤƤȿǤ롣
@@ -1426,9 +1721,35 @@
       return $this->update_if_modified($fv, $db);
     }
 
+    /**
+     * ˹礦褦updateʸ֤
+     *
+     * ʣԤѹԤ롣
+     * primary keyꤵƤʤƤ褤
+     *
+     * $fvϢͤĤ褦ʥ쥳ɤˤʤ褦
+     * updateʸ֤
+     * set_condition()ʤɤƤȿǤ롣
+     * 顼ʤɤͳŬڤSQLʸФʤ硢ʸ֤
+     *
+     *@see update()
+     *
+     *@access public
+     *@param string $fv ե̾򥭡ˤϢ
+     *@return string updateʸʸ󡣥顼ξʸ
+     */
+    
+    function update_all(&$fv) {
+      $db = "";
+      $modify_all = true;
+      return $this->update_if_modified($fv, $db, $modify_all);
+    }
+    
     /**
      * ˹礦褦ʡʬset褦updateʸ֤
      *
+     * primary keywhereǻꤵƤʤϥ顼
+     *
      * $fvϢͤĤ褦ʥ쥳ɤˤʤ褦ˡ
      * ƤʬΤ߹褦updateʸ֤
      * set_condition()ʤɤƤȿǤ롣
@@ -1436,20 +1757,37 @@
      *
      *@access public
      *@param string $fv ե̾򥭡ˤϢ
+     *@param string $row
+     *@param string $modify_all
      *@return string updateʸʸ󡣥顼ξʸ
      */
 
-    function update_if_modified(&$fv, &$db) {
+
+    function update_if_modified(&$fv, &$row, $modify_all=false) {
       if (!is_array($fv))            { $this->_debug2('update()'); return ""; }
       if (!$this->update_fields)     { $this->_debug2('update()'); return ""; }
 
-      $update_array = array();
+      // update_all()ǤʤȤˡ
+      // primary key¸ߤơ֥ǡ
+      // primary key whereǻꤵƤʤϥ顼
+      if(!$modify_all &&
+	 $this->pkey_name && 
+	 !$this->pkey_specified_in_where()) {
+
+	  $this->raiseError(SQL_ERROR_NO_PKEY_IN_WHERE, null, null,
+			    "all primary key of '$this->main_table' must be specified for this update",null);
+
+	  return "";
+      }
+      //updatefield¸ߤƤ뤫ɤΥå
+      $result = $this->field_exists_for_query(array_keys($fv), "update");
+      $update_array = array(); // updateɬפե
       reset($this->update_fields);
       while (list($k, $v) = each($this->update_fields)) {
 	//echo $k, $v, " - ", $fv[$v], "<br>";
 	if (isset($fv[$v])) {
 	  // updateоݤΥե
-	  if (is_object($db) && ((string)($fv[$v]) === $db->f($v))) {
+	  if (is_array($row) && ((string)($fv[$v]) === $row[$v])) {
 	    // äɬפϤʤ
 	  } else {
 	    $update_array[] = $v. "=". $this->var2sqlval($v, $fv[$v]);
@@ -1458,34 +1796,35 @@
       }
 
       if ($update_array) {
-	if (sizeof($this->where)) {
-	  //ͤȰäƤĤäƤɤȽǤ롩
-	  //$update_array[] = sprintf("n_click=0");
-	  //$update_array[] = sprintf("last_acc_id=0");
+	if (!$modify_all && sizeof($this->where) == 0) {
+	  // updateˤϡSQL֥ȤФ
+	  //expectError(SQL_ERROR_NO_WHERE_CLAUSE);
+	  //update();
+	  //popExpect();
+	  //Ȥơ顼Фʤ褦ˤ
+	  $this->raiseError(SQL_ERROR_NO_WHERE_CLAUSE,null,null,"No conditions given for update()",null);
+	}
 
-	  $from_tables = array();
-	  if (is_array($this->table) && sizeof($this->table) > 0) {
-	    // main_table 
-	    foreach ($this->table as $table) {
-	      if ($table != $this->main_table) {
-		$from_tables[] = $table;
-	      }
+	$from_tables = array(); // ʣtableǤupdate(PostgreSQL?)
+	if (is_array($this->table) && sizeof($this->table) > 0) {
+	  // main_table 
+	  foreach ($this->table as $table) {
+	    if ($table != $this->main_table) {
+	      $from_tables[] = $table;
 	    }
 	  }
+	}	  
 	  
-	  $sql = sprintf("update %s set %s",
-			 $this->main_table,
-			 implode(",", $update_array));
-	  if (sizeof($from_tables) > 0) {
-	    $sql .= " from ";
-	    $sql .= implode(",", $from_tables);
-	  }
+	$sql = sprintf("update %s set %s",
+		       $this->main_table,
+		       implode(",", $update_array));
+	if (sizeof($from_tables) > 0) {
+	  $sql .= " from ";
+	  $sql .= implode(",", $from_tables);
+	}
 
+	if ($this->where) {
 	  $sql .= sprintf(" where %s", implode(" and ", $this->where));
-
-	} else {
-	  // whereʤ񤭴Τϥ顼Ȥߤʤ٤
-	  $sql = "";
 	}
       } else {
 	// 񤭴Τʤʤ̵
@@ -1498,6 +1837,35 @@
       }
     }
 
+    /**
+     * primary key whereǻꤵƤ뤫å
+     *
+     * Ƥprimary keywhereǻꤵƤ뤫ɤ֤
+     * 1쥳ɤˤĤƤǤ뤳ȤꤹʤɤѤ롣
+     *
+     * primary key̵ơ֥ξˤϡ
+     * ˡꤵƤפȤϤߤʤ
+     * ꤵƤʤȤfalse֤
+     *
+     * @see update_if_modified
+     * @access private
+     * $return boolean pkeyꤵƤtrue
+     */
+    function pkey_specified_in_where(){
+      if(!$this->pkey_name) {
+	// primary key̵ơ֥ξ
+	return false;
+      } elseif (!is_array($this->pkey_name)) {
+	$this->pkey_name = array($this->pkey_name);
+      }
+
+      foreach ($this->pkey_name as $v) {
+	if (!isset($this->where[$v])) {
+	  return false;
+	}
+      }
+      return true;
+    }
     /**
      * delete records.
      *
@@ -1511,17 +1879,89 @@
       if (!$this->condition_check_delete()) { return ""; }
 
       if (sizeof($this->where)) {
-	$sql = sprintf("delete from \"%s\" where %s",
+	$sql = sprintf("delete from %s where %s",
 		       $this->main_table,
 		       implode(" and ", $this->where));
       } else {
-	// äΤϥ顼ˡ
-	$sql = "";
+	// deleteˤϡSQL֥ȤФ
+	//expectError(SQL_ERROR_NO_WHERE_CLAUSE);
+	//delete();
+	//popExpect();
+	//Ȥơ顼Фʤ褦ˤ
+	$this->raiseError(SQL_ERROR_NO_WHERE_CLAUSE,null,null,"No conditions given for delete()",null);
+	$sql = sprintf("delete from %s",$this->main_table);
       }
 
       return $sql;
     }
 
+    /**
+     * field ¸ߤĴ٤
+     *
+     * @param $field string set_conditionϤ줿field̾
+     * @return ¸ߤfield̾ξtrue¸ߤʤfalse֤
+     * @access public
+     */
+    function field_exists($field){
+      if (preg_match('/\./',$field)) {
+	//join¾Υơ֥set_condition硣ǥ顼ˤͽ
+	//$this->raiseError(SQL_ERROR_OTHERTABLE);
+	return true;
+      } elseif (!$this->field_type[$field]) {
+	$this->raiseError(SQL_ERROR_NOSUCHFIELD, null, null,
+			  "'$field' does not exist in '$this->main_table'",null);
+	return false;
+      } else {
+	return true;
+      }
+    }
+    /**
+     * insert/update򤹤Ȥˡfield ¸ߤĴ٤
+     *
+     * @param $field string set_conditionϤ줿field̾
+     * @param $query string insertupdate
+     * @return ¸ߤfield̾ξtrue¸ߤʤfalse֤
+     * @access private
+     */
+    function field_exists_for_query($field,$query){
+      if ($query == "insert") {
+	$array =& $this->insert_fields;
+	$operation = "inserting";
+      } elseif ($query == "update") {
+	$array =& $this->update_fields;
+	$operation = "updating";
+      } else {
+	return false;
+      }
+
+      if (!is_array($field)) {
+	$field = array($field);
+      }
+
+      $error_keys = array();
+      if (sizeof($field) == 0) {
+	$error_occured = false;
+      } else {
+	$error_occured = true;
+	foreach ($field as $key=>$value) {
+	  if (in_array($value, $array)) {
+	    $error_occured = false;
+	  } else {
+	    $error_keys[] = $value;
+	  }
+	}
+      }
+      
+      if ($error_occured) {
+	$this->raiseError(SQL_ERROR_NOSUCHFIELD, null, null,
+			  "Illegal field name for $operation '$this->main_table' : '".
+			  implode("','", $error_keys)."'",null);
+	return false;
+      } else {
+	return true;
+      }
+    }
+
     /**
      * Ϳ줿SQLɤɤ֤
      *
@@ -2041,7 +2481,7 @@
     }
     function _debug2 ($fnname) {
       if ($this->debug) {
-	echo "<B>$fname</B><BR>";
+	echo "<B>$fnname</B><BR>";
       }
     }
   }
@@ -2053,4 +2493,98 @@
     }
   };
 
+  // {{{ class SQL_Error
+  /**
+   * SQL˴ؤ륨顼åϤ륯饹
+   *
+   */
+  class SQL_Error extends PEAR_Error
+    {
+      // {{{ constructor
+      /**
+       * SQL_Error constructor.
+       *
+       * @param mixed   $code   SQL error code, or string with error message.
+       * @param integer $mode   what "error mode" to operate in
+       * @param integer $level  what error level to use for $mode & PEAR_ERROR_TRIGGER
+       * @param mixed   $debuginfo  additional debug info, such as the last query
+       *
+       * @access public
+       *
+       * @see PEAR_Error
+       */
+      
+      function SQL_Error($code = SQL_ERROR, $mode = PEAR_ERROR_RETURN,
+			$level = E_USER_NOTICE, $debuginfo = null)
+	{
+	  if (is_int($code)) {
+            $this->PEAR_Error('SQL Error: ' . $this->errorMessage($code), $code, $mode, $level, $debuginfo);
+	  } else {
+            $this->PEAR_Error("SQL Error: $code", SQL_ERROR, $mode, $level, $debuginfo);
+	  }	  
+	}
+      // }}}
+
+      // }}}
+      // {{{ errorMessage()
+      /**
+       * Return a textual error message for a SQL error code
+       *
+       * @param integer $value error code
+       *
+       * @return string error message, or false if the error code was
+       * not recognized
+       */
+      function errorMessage($value)
+	{
+	  static $errorMessages;
+	  if (!isset($errorMessages)) {
+            $errorMessages = array(
+				   SQL_ERROR                    => 'unknown error',
+				   SQL_ERROR_ALREADY_EXISTS     => 'already exists',
+				   SQL_ERROR_CANNOT_CREATE      => 'can not create',
+				   SQL_ERROR_CANNOT_DELETE      => 'can not delete',
+				   SQL_ERROR_CANNOT_DROP        => 'can not drop',
+				   SQL_ERROR_CONSTRAINT         => 'constraint violation',
+				   SQL_ERROR_DIVZERO            => 'division by zero',
+				   SQL_ERROR_INVALID            => 'invalid',
+				   SQL_ERROR_INVALID_DATE       => 'invalid date or time',
+				   SQL_ERROR_INVALID_NUMBER     => 'invalid number',
+				   SQL_ERROR_MISMATCH           => 'mismatch',
+				   SQL_ERROR_NODBSELECTED       => 'no database selected',
+				   SQL_ERROR_NOSUCHFIELD        => 'no such field',
+				   SQL_ERROR_NOSUCHTABLE        => 'no such table',
+				   SQL_ERROR_NOT_CAPABLE        => 'SQL backend not capable',
+				   SQL_ERROR_NOT_FOUND          => 'not found',
+				   SQL_ERROR_NOT_LOCKED         => 'not locked',
+				   SQL_ERROR_SYNTAX             => 'syntax error',
+				   SQL_ERROR_UNSUPPORTED        => 'not supported',
+				   SQL_ERROR_VALUE_COUNT_ON_ROW => 'value count on row',
+				   SQL_ERROR_INVALID_DSN        => 'invalid DSN',
+				   SQL_ERROR_CONNECT_FAILED     => 'connect failed',
+				   SQL_OK                       => 'no error',
+				   SQL_WARNING                  => 'unknown warning',
+				   SQL_WARNING_READ_ONLY        => 'read only',
+				   SQL_ERROR_NEED_MORE_DATA     => 'insufficient data supplied',
+				   SQL_ERROR_EXTENSION_NOT_FOUND=> 'extension not found',
+				   SQL_ERROR_NOSUCHDB           => 'no such database',
+				   SQL_ERROR_ACCESS_VIOLATION   => 'insufficient permissions',
+				   SQL_ERROR_TRUNCATED          => 'truncated',
+				   SQL_ERROR_NOCONDITIONS       => 'no conditions given',
+				   SQL_ERROR_NOSETVAR           => 'no set variables',
+				   SQL_ERROR_OTHERTABLE         => 'access to other table',
+				   SQL_ERROR_NO_WHERE_CLAUSE    => 'no where clause given',
+				   SQL_ERROR_NO_PKEY_IN_WHERE	=> 'the primary key is not in where clause',
+				   );
+	  }
+	  
+	  if (PEAR::isError($value)) {
+            $value = $value->getCode();
+	  }
+	  
+	  return isset($errorMessages[$value]) ? $errorMessages[$value] : $errorMessages[SQL_ERROR];
+	}
+      
+    }
+  // }}}
 }?>
