@@ -4,7 +4,7 @@
  * Copyright (c) 1998 by Jay Bloodworth
  * This file is modified by Dino Co.,Ltd .
  *
- * $Id: of_text.inc.php,v 1.5 2003/04/20 12:02:59 tumf Exp $
+ * $Id: of_text.inc.php,v 1.6 2003/08/24 09:02:09 hanawa Exp $
  */
 
 
@@ -56,7 +56,7 @@ class of_text extends of_element {
     return $str;
   }
 
-  function self_get_frozen($val,$which, &$count) {
+  function self_get_frozen($val,$which, &$count, $hidden = false) {
     $str = "";
     if (is_array($this->value))
       $v = $this->value[$which];
@@ -67,14 +67,18 @@ class of_text extends of_element {
     $n = $this->name . ($this->multiple ? "[]" : "");
 //    $v = str_replace('"', '&quot;', $v);
     $v = htmlspecialchars($v, ENT_QUOTES);
-    $str .= "<input type='hidden' name='$n' value=\"$v\" />\n";
+    $str .= "<input type='hidden' name='$n' value=\"$v\"\n/>";
     // $str .= "<table border=1><tr><td>$v</td></tr></table>\n";
-    if($this->pass){
+
+    if ($hidden) {
+      // never output *visible* HTML
+    } elseif($this->pass){
       $str .= "***********";
     }else{
       $str .= $v;
+      $str .="\n";
     }
-    $str .="\n";
+
     $count = 1;
     return $str;
   }
