@@ -6,7 +6,7 @@
  * Copyright (c) 1998 by Jay Bloodworth
  * This file is modified by Dino Co.,Ltd.
  * 
- * $Id: oohforms.inc.php,v 1.13 2003/06/27 03:41:36 hanawa Exp $
+ * $Id: oohforms.inc.php,v 1.16 2003/09/19 02:40:58 hanawa Exp $
  */
 
 /**
@@ -38,7 +38,11 @@ class of_element {
     return $count;
   }
 
-  function self_get_frozen($val, $which, &$count) {
+  function self_get_hidden($val, $which, &$count) {
+    return $this->self_get_frozen($val, $which, &$count, true);
+  }
+
+  function self_get_frozen($val, $which, &$count, $hidden = false) {
     return $this->self_get($val, $which, $count);
   }
 
@@ -201,8 +205,9 @@ class form {
     
     if ($this->hidden) {
       reset($this->hidden);
-      while (list($k,$elname) = each($this->hidden)) 
+      while (list($k,$elname) = each($this->hidden)) {
         $str .= $this->get_element($elname);
+      }
     }
     if (is_object($sess) && ($sess->mode == "get")) {
       $str .= sprintf("<input type=\"hidden\" name=\"%s\" value=\"%s\">\n", $sess->name, $sess->id);
@@ -290,7 +295,7 @@ function check_submitting_once() {
     $str = "";
     $x   = 0;
     $flag_nametranslation = false;
-    
+
     // see add_element: translate $foo[int] to $foo{int}
 #   Original pattern
 #   if (preg_match("/(w+)\[(\d+)\]/i", $name, $regs) {
@@ -299,7 +304,7 @@ function check_submitting_once() {
        $name = sprintf("%s{%s}", $regs[1], $regs[2]);
        $flag_nametranslation = true;
     }
-    
+
     if (!isset($this->elements[$name])) 
       return false; 
 
@@ -315,7 +320,11 @@ function check_submitting_once() {
 
     if ($this->elements[$name]["frozen"]){
       if(preg_match("/^of_.*$/",get_class($el))){
-	$str .= $el->self_get_frozen($value,$this->elements[$name]["which"], $x);
+	if ($this->elements[$name]["frozen"] == 1){
+	  $str .= $el->self_get_frozen($value,$this->elements[$name]["which"], $x);
+	} else {
+	  $str .= $el->self_get_hidden($value,$this->elements[$name]["which"], $x);
+	}
       }
     }else{
       if(preg_match("/^of_.*$/",get_class($el))){
@@ -376,6 +385,7 @@ function check_submitting_once() {
       reset($this->elements);
       $elrec = current($this->elements);
     }
+
     while ($elrec) {
       $el = $elrec["ob"];
       if(preg_match("/^of_.*$/",get_class($el))){
@@ -392,7 +402,15 @@ function check_submitting_once() {
     }
   }
 
-  function freeze($flist="") {
+  function hide($flist="") {
+    $this->freeze($flist, 2);
+  }
+
+  function freeze($flist="", $freeze_type = 1) {
+    if ($flist && !is_array($flist)) {
+      $flist = array($flist);
+    }
+
     if ($flist) {
       reset($flist);
       $elrec = $this->elements[current($flist)];
@@ -402,7 +420,8 @@ function check_submitting_once() {
     }
     while ($elrec) {
       $el = $elrec["ob"];
-      $this->elements[$el->name]["frozen"]=1;
+      // $freeze_type -> 0: not-frozen , 1: frozen , 2: hidden
+      $this->elements[$el->name]["frozen"]=$freeze_type;
       if ($flist) {
         next($flist);
         $elrec = $this->elements[current($flist)];
@@ -415,12 +434,13 @@ function check_submitting_once() {
 
 } /* end FORM */
 
-require_once("of_text.inc.php");
-require_once("of_select.inc.php");
-require_once("of_radio.inc.php");
+require_once(DINOLIB_DIR . "/form/tpl_form.inc.php");
+require_once(DINOLIB_DIR . "/form/of_text.inc.php");
+require_once(DINOLIB_DIR . "/form/of_select.inc.php");
+require_once(DINOLIB_DIR . "/form/of_radio.inc.php");
 //require_once("of_radio2.inc.php");
-require_once("of_checkbox.inc.php");
-require_once("of_textarea.inc.php");
-require_once("of_file.inc.php");
+require_once(DINOLIB_DIR . "/form/of_checkbox.inc.php");
+require_once(DINOLIB_DIR . "/form/of_textarea.inc.php");
+require_once(DINOLIB_DIR . "/form/of_file.inc.php");
 
 ?>
