@@ -5,7 +5,7 @@
    *   PukiWiki(http:// pukiwiki.org/) ̤ήѤƤޤ
    *   PukiWiki Copyright (C) 2001,2002 PukiWiki Developers Team,sng.
    *            License is GNU/GPL.
-   *   $Id: page.php,v 1.12 2003/04/16 07:42:37 tumf Exp $
+   *   $Id: page.php,v 1.13 2003/06/03 10:54:03 tumf Exp $
    *   This file is modified by Dino Co.,Ltd.
    */
 
@@ -18,9 +18,12 @@
   require_once("link.php");
 
   class page {
+    var $class_name="page";
     var $src;
     var $page_list=array();
 
+    var $foot_explain;
+    var $note_id;
     var $note_hr = '<hr />';
     var $strip_link_wall=true;
 
@@ -103,8 +106,6 @@
     
       // 
       function make_note($str){
-	global $note_id,$foot_explain;
-
 	$str = preg_replace("/^\(\(/","",$str);
 	$str = preg_replace("/\s*\)\)$/","",$str);
 
@@ -112,9 +113,20 @@
 
 	//$str = make_user_rules($str);
 
-	$foot_explain[] = "<a name=\"notefoot_$note_id\" href=\"#notetext_$note_id\" class=\"note_super\">*$note_id</a> <span class=\"small\">$str</span><br />\n";
-	$note =  "<a name=\"notetext_$note_id\" href=\"#notefoot_$note_id\" class=\"note_super\">*$note_id</a>";
-	$note_id++;
+	$this->foot_explain[] 
+	  = sprintf("<a name=\"notefoot_%s\" href=\"#notetext_%s\" class=\"note_super\">*%s</a> <span class=\"small\">$str</span><br />\n",
+		    $this->note_id,
+		    $this->note_id,
+		    $this->note_id
+		    );
+		    
+	$note
+	  = sprintf("<a name=\"notetext_%s\" href=\"#notefoot_%s\" class=\"note_super\">*%s</a>",
+			 $this->note_id,
+			 $this->note_id,
+			 $this->note_id
+			 );
+	$this->note_id++;
 
 	return $note;
       }
@@ -469,10 +481,9 @@
 
 	$str = join("\n", $result_last);
 
-	global $foot_explain;
-	if ($foot_explain){
+	if ($this->foot_explain){
 	  $str .= "\n<div class=\"footnote\">\n$this->note_hr\n";
-	  $str .= join("\n",$this->inline2($foot_explain));
+	  $str .= join("\n",$this->inline2($this->foot_explain));
 	  $str .= "\n</div>\n";
 	}
 
