@@ -6,7 +6,7 @@
    * Smarty plugin
    * ----------------------------------------
    * File:    function.dino_tpl_form.php
-   * Id  :    $Id: function.dino_tpl_form.php,v 1.5 2003/06/17 10:13:03 tumf Exp $
+   * Id  :    $Id: function.dino_tpl_form.php,v 1.12 2003/10/28 14:19:14 sato_t Exp $
    * Type:    function
    * Name:    tpl_form
    * Purpose: ƥץ졼ˡtmpl_formǤФե४֥Ȥ
@@ -31,7 +31,7 @@
     if(strlen($params["form_name"])>0){
       $form_name=$params["form_name"];
     }
-    $f = $smarty->_tpl_vars['__form_obj'][$form_name];
+    $f =& $smarty->_tpl_vars['__form_obj'][$form_name];
     if($f) {
       if($params['type'] == 'start') {
 	print $f->get_start();
@@ -43,10 +43,36 @@
       }
 
       $value = false;
-      if($params['value']) 
-	$value = $params['value'];
+      if (isset($params['value'])) {
+	$value = (string)$params['value'];
+      }
 
-      $element=$f->get_element($params['name'], $value);
+      $single = false;
+      if (isset($params['single'])){
+	$single = true;
+      }
+
+      if($params['type'] == 'group-begin') {
+	if (isset($params['name'])){
+	  $f->set_group_flag(true);
+	  $f->set_now_group($params['name']);
+	  $f->increment_counter($params['name']);
+	} else{
+	  //顼
+	}
+      }
+      if($params['type'] == 'group-end') {
+	$f->set_group_flag(false);
+      }
+
+      if ($params['status'] == 'frozen') {
+	$f->freeze($params['name']);
+      } elseif ($params['status'] == 'hidden') {
+	$f->hide($params['name']);
+      } elseif ($params['status'] == 'show') {
+	$f->show($params['name']);
+      } 
+      $element=$f->get_element($params['name'], $value, $single);
 
       if($params['escape']=="html"){
 	print htmlspecialchars($element);
